#
# This file is part of rhevsh. rhevsh is free software that is made
# available under the MIT license. Consult the file "LICENSE" that is
# distributed together with this file for the exact licensing terms.
#
# rhevsh is copyright (c) 2011 by the rhevsh authors. See the file
# "AUTHORS" for a complete overview.

import sys
import textwrap

from cli.command import *
from cli.context import ExecutionContext

import rhev
from rhev import schema

from rhevsh.settings import RhevshSettings
from rhevsh.command import *
from rhevsh.format import *
from rhevsh.object import create


class RhevshExecutionContext(ExecutionContext):

    Settings = RhevshSettings

    name = 'rhevsh'
    welcome = textwrap.dedent("""\
        Welcome to Red Hat Enterprise Virtualization.

        Type:  'help' for help with commands
               'exit' to leave this interactive shell
        """)

    REMOTE_ERROR = 10
    NOT_FOUND = 11

    def __init__(self):
        super(RhevshExecutionContext, self).__init__()
        self.connection = None
        self.formatter = create(Formatter, self.settings['rhevsh:output_format'])
        self.settings.add_callback('cli:verbosity', self._set_verbosity)
        self.settings.add_callback('rhevsh:output_format', self._set_formatter)

    def _set_verbosity(self, key, value):
        if self.connection is None:
            return
        if value:
            level = 10
        else:
            level = 1
        self.connection.verbosity = level

    def _set_formatter(self, key, value):
        self.formatter = create(Formatter, value)

    def _handle_exception(self, e):
        if isinstance(e, rhev.Error):
            msg = 'error: rhev: %s\n' % str(e)
            if hasattr(e, 'detail'):
                msg += 'detail: %s\n' % e.detail
            sys.stderr.write(msg)
            self.status = self.REMOTE_ERROR
        else:
            super(RhevshExecutionContext, self)._handle_exception(e)

    def setup_commands(self):
        self.add_command(SetCommand)
        self.add_command(SaveCommand)
        self.add_command(CdCommand)
        self.add_command(PwdCommand)
        self.add_command(ClearCommand)
        self.add_command(ExitCommand)
        self.add_command(ActionCommand)
        self.add_command(CreateCommand)
        self.add_command(ConnectCommand)
        self.add_command(ConsoleCommand)
        self.add_command(CreateCommand)
        self.add_command(DeleteCommand)
        self.add_command(DisconnectCommand)
        self.add_command(HelpCommand)
        self.add_command(ListCommand)
        self.add_command(PingCommand)
        self.add_command(ShowCommand)
        self.add_command(StatusCommand)
        self.add_command(UpdateCommand)

    def _get_prompt_variables(self):
        """Return a dict with prompt variables."""
        subst = {}
        if self.connection:
            api = self.connection.api()
            version = api.product_info.version
            subst['version'] = '%s.%s' % (version.major, version.minor)
        else:
            subst['version'] = ''
        return subst

    def _set_prompt(self):
        """Update the prompt."""
        if self.connection is None:
            prompt = self.settings['rhevsh:ps1.disconnected']
        else:
            subst = self._get_prompt_variables()
            prompt = self.settings['rhevsh:ps1.connected'] % subst
        self.settings['cli:ps1'] = prompt

    def _read_command(self):
        self._set_prompt()
        return super(RhevshExecutionContext, self)._read_command()
